@@ -1,12 +1,9 @@
 """
 Classes for managing Checkpoints.
 """
-
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-
 from tornado.web import HTTPError
-
 from traitlets.config.configurable import LoggingConfigurable
 
 
@@ -22,38 +19,39 @@ class Checkpoints(LoggingConfigurable):
     delete_checkpoint(self, checkpoint_id, path)
     list_checkpoints(self, path)
     """
+
     def create_checkpoint(self, contents_mgr, path):
         """Create a checkpoint."""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def restore_checkpoint(self, contents_mgr, checkpoint_id, path):
         """Restore a checkpoint"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def rename_checkpoint(self, checkpoint_id, old_path, new_path):
         """Rename a single checkpoint from old_path to new_path."""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def delete_checkpoint(self, checkpoint_id, path):
         """delete a checkpoint for a file"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def list_checkpoints(self, path):
         """Return a list of checkpoints for a given file"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def rename_all_checkpoints(self, old_path, new_path):
         """Rename all checkpoints for old_path to new_path."""
         for cp in self.list_checkpoints(old_path):
-            self.rename_checkpoint(cp['id'], old_path, new_path)
+            self.rename_checkpoint(cp["id"], old_path, new_path)
 
     def delete_all_checkpoints(self, path):
         """Delete all checkpoints for the given path."""
         for checkpoint in self.list_checkpoints(path):
-            self.delete_checkpoint(checkpoint['id'], path)
+            self.delete_checkpoint(checkpoint["id"], path)
 
 
-class GenericCheckpointsMixin(object):
+class GenericCheckpointsMixin:
     """
     Helper for creating Checkpoints subclasses that can be used with any
     ContentsManager.
@@ -77,30 +75,30 @@ class GenericCheckpointsMixin(object):
 
     def create_checkpoint(self, contents_mgr, path):
         model = contents_mgr.get(path, content=True)
-        type = model['type']
-        if type == 'notebook':
+        type_ = model["type"]
+        if type_ == "notebook":
             return self.create_notebook_checkpoint(
-                model['content'],
+                model["content"],
                 path,
             )
-        elif type == 'file':
+        elif type_ == "file":
             return self.create_file_checkpoint(
-                model['content'],
-                model['format'],
+                model["content"],
+                model["format"],
                 path,
             )
         else:
-            raise HTTPError(500, u'Unexpected type %s' % type)
+            raise HTTPError(500, "Unexpected type %s" % type)
 
     def restore_checkpoint(self, contents_mgr, checkpoint_id, path):
         """Restore a checkpoint."""
-        type = contents_mgr.get(path, content=False)['type']
-        if type == 'notebook':
+        type_ = contents_mgr.get(path, content=False)["type"]
+        if type_ == "notebook":
             model = self.get_notebook_checkpoint(checkpoint_id, path)
-        elif type == 'file':
+        elif type_ == "file":
             model = self.get_file_checkpoint(checkpoint_id, path)
         else:
-            raise HTTPError(500, u'Unexpected type %s' % type)
+            raise HTTPError(500, "Unexpected type %s" % type_)
         contents_mgr.save(model, path)
 
     # Required Methods
@@ -109,14 +107,14 @@ class GenericCheckpointsMixin(object):
 
         Returns a checkpoint model for the new checkpoint.
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def create_notebook_checkpoint(self, nb, path):
         """Create a checkpoint of the current state of a file
 
         Returns a checkpoint model for the new checkpoint.
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def get_file_checkpoint(self, checkpoint_id, path):
         """Get the content of a checkpoint for a non-notebook file.
@@ -128,7 +126,7 @@ class GenericCheckpointsMixin(object):
             'format': {'text','base64'},
         }
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     def get_notebook_checkpoint(self, checkpoint_id, path):
         """Get the content of a checkpoint for a notebook.
@@ -139,42 +137,43 @@ class GenericCheckpointsMixin(object):
             'content': <output of nbformat.read>,
         }
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
 
 class AsyncCheckpoints(Checkpoints):
     """
     Base class for managing checkpoints for a ContentsManager asynchronously.
     """
+
     async def create_checkpoint(self, contents_mgr, path):
         """Create a checkpoint."""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def restore_checkpoint(self, contents_mgr, checkpoint_id, path):
         """Restore a checkpoint"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def rename_checkpoint(self, checkpoint_id, old_path, new_path):
         """Rename a single checkpoint from old_path to new_path."""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def delete_checkpoint(self, checkpoint_id, path):
         """delete a checkpoint for a file"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def list_checkpoints(self, path):
         """Return a list of checkpoints for a given file"""
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def rename_all_checkpoints(self, old_path, new_path):
         """Rename all checkpoints for old_path to new_path."""
-        for cp in (await self.list_checkpoints(old_path)):
-            await self.rename_checkpoint(cp['id'], old_path, new_path)
+        for cp in await self.list_checkpoints(old_path):
+            await self.rename_checkpoint(cp["id"], old_path, new_path)
 
     async def delete_all_checkpoints(self, path):
         """Delete all checkpoints for the given path."""
-        for checkpoint in (await self.list_checkpoints(path)):
-            await self.delete_checkpoint(checkpoint['id'], path)
+        for checkpoint in await self.list_checkpoints(path):
+            await self.delete_checkpoint(checkpoint["id"], path)
 
 
 class AsyncGenericCheckpointsMixin(GenericCheckpointsMixin):
@@ -185,30 +184,31 @@ class AsyncGenericCheckpointsMixin(GenericCheckpointsMixin):
 
     async def create_checkpoint(self, contents_mgr, path):
         model = await contents_mgr.get(path, content=True)
-        type = model['type']
-        if type == 'notebook':
+        type_ = model["type"]
+        if type_ == "notebook":
             return await self.create_notebook_checkpoint(
-                model['content'],
+                model["content"],
                 path,
             )
-        elif type == 'file':
+        elif type_ == "file":
             return await self.create_file_checkpoint(
-                model['content'],
-                model['format'],
+                model["content"],
+                model["format"],
                 path,
             )
         else:
-            raise HTTPError(500, u'Unexpected type %s' % type)
+            raise HTTPError(500, "Unexpected type %s" % type_)
 
     async def restore_checkpoint(self, contents_mgr, checkpoint_id, path):
         """Restore a checkpoint."""
-        type = await contents_mgr.get(path, content=False)['type']
-        if type == 'notebook':
+        content_model = await contents_mgr.get(path, content=False)
+        type_ = content_model["type"]
+        if type_ == "notebook":
             model = await self.get_notebook_checkpoint(checkpoint_id, path)
-        elif type == 'file':
+        elif type_ == "file":
             model = await self.get_file_checkpoint(checkpoint_id, path)
         else:
-            raise HTTPError(500, u'Unexpected type %s' % type)
+            raise HTTPError(500, "Unexpected type %s" % type_)
         await contents_mgr.save(model, path)
 
     # Required Methods
@@ -217,14 +217,14 @@ class AsyncGenericCheckpointsMixin(GenericCheckpointsMixin):
 
         Returns a checkpoint model for the new checkpoint.
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def create_notebook_checkpoint(self, nb, path):
         """Create a checkpoint of the current state of a file
 
         Returns a checkpoint model for the new checkpoint.
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def get_file_checkpoint(self, checkpoint_id, path):
         """Get the content of a checkpoint for a non-notebook file.
@@ -236,7 +236,7 @@ class AsyncGenericCheckpointsMixin(GenericCheckpointsMixin):
             'format': {'text','base64'},
         }
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
 
     async def get_notebook_checkpoint(self, checkpoint_id, path):
         """Get the content of a checkpoint for a notebook.
@@ -247,4 +247,4 @@ class AsyncGenericCheckpointsMixin(GenericCheckpointsMixin):
             'content': <output of nbformat.read>,
         }
         """
-        raise NotImplementedError("must be implemented in a subclass")
+        raise NotImplementedError
