@@ -1,28 +1,167 @@
 """A base class session manager."""
+
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+import os
 import pathlib
 import uuid
+from typing import Any, Dict, List, NewType, Optional, Union
+
+KernelName = NewType("KernelName", str)
+ModelName = NewType("ModelName", str)
 
 try:
     import sqlite3
 except ImportError:
     # fallback on pysqlite2 if Python was build without sqlite
-    from pysqlite2 import dbapi2 as sqlite3
+    from pysqlite2 import dbapi2 as sqlite3  # type:ignore
 
+from dataclasses import dataclass, fields
+
+from jupyter_core.utils import ensure_async
 from tornado import web
-
+from traitlets import Instance, TraitError, Unicode, validate
 from traitlets.config.configurable import LoggingConfigurable
-from traitlets import Instance
-from traitlets import Unicode
-from traitlets import validate
-from traitlets import TraitError
 
-from jupyter_server.utils import ensure_async
 from jupyter_server.traittypes import InstanceFromClasses
 
 
+class KernelSessionRecordConflict(Exception):
+    """Exception class to use when two KernelSessionRecords cannot
+    merge because of conflicting data.
+    """
+
+    pass
+
+
+@dataclass
+class KernelSessionRecord:
+    """A record object for tracking a Jupyter Server Kernel Session.
+
+    Two records that share a session_id must also share a kernel_id, while
+    kernels can have multiple session (and thereby) session_ids
+    associated with them.
+    """
+
+    session_id: Optional[str] = None
+    kernel_id: Optional[str] = None
+
+    def __eq__(self, other: object) -> bool:
+        """Whether a record equals another."""
+        if isinstance(other, KernelSessionRecord):
+            condition1 = self.kernel_id and self.kernel_id == other.kernel_id
+            condition2 = all(
+                [
+                    self.session_id == other.session_id,
+                    self.kernel_id is None or other.kernel_id is None,
+                ]
+            )
+            if any([condition1, condition2]):
+                return True
+            # If two records share session_id but have different kernels, this is
+            # and ill-posed expression. This should never be true. Raise an exception
+            # to inform the user.
+            if all(
+                [
+                    self.session_id,
+                    self.session_id == other.session_id,
+                    self.kernel_id != other.kernel_id,
+                ]
+            ):
+                msg = (
+                    "A single session_id can only have one kernel_id "
+                    "associated with. These two KernelSessionRecords share the same "
+                    "session_id but have different kernel_ids. This should "
+                    "not be possible and is likely an issue with the session "
+                    "records."
+                )
+                raise KernelSessionRecordConflict(msg)
+        return False
+
+    def update(self, other: "KernelSessionRecord") -> None:
+        """Updates in-place a kernel from other (only accepts positive updates"""
+        if not isinstance(other, KernelSessionRecord):
+            msg = "'other' must be an instance of KernelSessionRecord."
+            raise TypeError(msg)
+
+        if other.kernel_id and self.kernel_id and other.kernel_id != self.kernel_id:
+            msg = "Could not update the record from 'other' because the two records conflict."
+            raise KernelSessionRecordConflict(msg)
+
+        for field in fields(self):
+            if hasattr(other, field.name) and getattr(other, field.name):
+                setattr(self, field.name, getattr(other, field.name))
+
+
+class KernelSessionRecordList:
+    """An object for storing and managing a list of KernelSessionRecords.
+
+    When adding a record to the list, the KernelSessionRecordList
+    first checks if the record already exists in the list. If it does,
+    the record will be updated with the new information; otherwise,
+    it will be appended.
+    """
+
+    _records: List[KernelSessionRecord]
+
+    def __init__(self, *records: KernelSessionRecord):
+        """Initialize a record list."""
+        self._records = []
+        for record in records:
+            self.update(record)
+
+    def __str__(self):
+        """The string representation of a record list."""
+        return str(self._records)
+
+    def __contains__(self, record: Union[KernelSessionRecord, str]) -> bool:
+        """Search for records by kernel_id and session_id"""
+        if isinstance(record, KernelSessionRecord) and record in self._records:
+            return True
+
+        if isinstance(record, str):
+            for r in self._records:
+                if record in [r.session_id, r.kernel_id]:
+                    return True
+        return False
+
+    def __len__(self):
+        """The length of the record list."""
+        return len(self._records)
+
+    def get(self, record: Union[KernelSessionRecord, str]) -> KernelSessionRecord:
+        """Return a full KernelSessionRecord from a session_id, kernel_id, or
+        incomplete KernelSessionRecord.
+        """
+        if isinstance(record, str):
+            for r in self._records:
+                if record == r.kernel_id or record == r.session_id:
+                    return r
+        elif isinstance(record, KernelSessionRecord):
+            for r in self._records:
+                if record == r:
+                    return record
+        msg = f"{record} not found in KernelSessionRecordList."
+        raise ValueError(msg)
+
+    def update(self, record: KernelSessionRecord) -> None:
+        """Update a record in-place or append it if not in the list."""
+        try:
+            idx = self._records.index(record)
+            self._records[idx].update(record)
+        except ValueError:
+            self._records.append(record)
+
+    def remove(self, record: KernelSessionRecord) -> None:
+        """Remove a record if its found in the list. If it's not found,
+        do nothing.
+        """
+        if record in self._records:
+            self._records.remove(record)
+
+
 class SessionManager(LoggingConfigurable):
+    """A session manager."""
 
     database_filepath = Unicode(
         default_value=":memory:",
@@ -36,6 +175,7 @@ class SessionManager(LoggingConfigurable):
 
     @validate("database_filepath")
     def _validate_database_filepath(self, proposal):
+        """Validate a database file path."""
         value = proposal["value"]
         if value == ":memory:":
             return value
@@ -43,15 +183,15 @@ class SessionManager(LoggingConfigurable):
         if path.exists():
             # Verify that the database path is not a directory.
             if path.is_dir():
-                raise TraitError(
-                    "`database_filepath` expected a file path, but the given path is a directory."
-                )
+                msg = "`database_filepath` expected a file path, but the given path is a directory."
+                raise TraitError(msg)
             # Verify that database path is an SQLite 3 Database by checking its header.
             with open(value, "rb") as f:
                 header = f.read(100)
 
             if not header.startswith(b"SQLite format 3") and not header == b"":
-                raise TraitError("The given file is not an SQLite database file.")
+                msg = "The given file is not an SQLite database file."
+                raise TraitError(msg)
         return value
 
     kernel_manager = Instance("jupyter_server.services.kernels.kernelmanager.MappingKernelManager")
@@ -62,6 +202,11 @@ class SessionManager(LoggingConfigurable):
         ]
     )
 
+    def __init__(self, *args, **kwargs):
+        """Initialize a record list."""
+        super().__init__(*args, **kwargs)
+        self._pending_sessions = KernelSessionRecordList()
+
     # Session database initialized below
     _cursor = None
     _connection = None
@@ -113,32 +258,93 @@ class SessionManager(LoggingConfigurable):
                 exists = True
         return exists
 
-    def new_session_id(self):
-        "Create a uuid for a new session"
+    def new_session_id(self) -> str:
+        """Create a uuid for a new session"""
         return str(uuid.uuid4())
 
     async def create_session(
-        self, path=None, name=None, type=None, kernel_name=None, kernel_id=None
-    ):
-        """Creates a session and returns its model"""
+        self,
+        path: Optional[str] = None,
+        name: Optional[ModelName] = None,
+        type: Optional[str] = None,
+        kernel_name: Optional[KernelName] = None,
+        kernel_id: Optional[str] = None,
+    ) -> Dict[str, Any]:
+        """Creates a session and returns its model
+
+        name: ModelName(str)
+            Usually the model name, like the filename associated with current
+            kernel.
+
+
+        """
         session_id = self.new_session_id()
+        record = KernelSessionRecord(session_id=session_id)
+        self._pending_sessions.update(record)
         if kernel_id is not None and kernel_id in self.kernel_manager:
             pass
         else:
             kernel_id = await self.start_kernel_for_session(
                 session_id, path, name, type, kernel_name
             )
+        record.kernel_id = kernel_id
+        self._pending_sessions.update(record)
         result = await self.save_session(
             session_id, path=path, name=name, type=type, kernel_id=kernel_id
         )
+        self._pending_sessions.remove(record)
         return result
 
-    async def start_kernel_for_session(self, session_id, path, name, type, kernel_name):
-        """Start a new kernel for a given session."""
+    def get_kernel_env(
+        self, path: Optional[str], name: Optional[ModelName] = None
+    ) -> Dict[str, str]:
+        """Return the environment variables that need to be set in the kernel
+
+        path : str
+            the url path for the given session.
+        name: ModelName(str), optional
+            Here the name is likely to be the name of the associated file
+            with the current kernel at startup time.
+
+
+        """
+        if name is not None:
+            cwd = self.kernel_manager.cwd_for_path(path)
+            path = os.path.join(cwd, name)
+        assert isinstance(path, str)
+        return {**os.environ, "JPY_SESSION_NAME": path}
+
+    async def start_kernel_for_session(
+        self,
+        session_id: str,
+        path: Optional[str],
+        name: Optional[ModelName],
+        type: Optional[str],
+        kernel_name: Optional[KernelName],
+    ) -> str:
+        """Start a new kernel for a given session.
+
+        session_id : str
+            uuid for the session; this method must be given a session_id
+        path : str
+            the path for the given session - seem to be a session id sometime.
+        name : str
+            Usually the model name, like the filename associated with current
+            kernel.
+        type : str
+            the type of the session
+        kernel_name : str
+            the name of the kernel specification to use.  The default kernel name will be used if not provided.
+
+        """
         # allow contents manager to specify kernels cwd
-        kernel_path = self.contents_manager.get_kernel_path(path=path)
+        kernel_path = await ensure_async(self.contents_manager.get_kernel_path(path=path))
+
+        kernel_env = self.get_kernel_env(path, name)
         kernel_id = await self.kernel_manager.start_kernel(
-            path=kernel_path, kernel_name=kernel_name
+            path=kernel_path,
+            kernel_name=kernel_name,
+            env=kernel_env,
         )
         return kernel_id
 
@@ -155,9 +361,9 @@ class SessionManager(LoggingConfigurable):
             uuid for the session; this method must be given a session_id
         path : str
             the path for the given session
-        name: str
+        name : str
             the name of the session
-        type: string
+        type : str
             the type of the session
         kernel_id : str
             a uuid for the kernel associated with this session
@@ -193,12 +399,14 @@ class SessionManager(LoggingConfigurable):
             session described by the kwarg.
         """
         if not kwargs:
-            raise TypeError("must specify a column to query")
+            msg = "must specify a column to query"
+            raise TypeError(msg)
 
         conditions = []
         for column in kwargs.keys():
             if column not in self._columns:
-                raise TypeError("No such column: %r", column)
+                msg = f"No such column: {column}"
+                raise TypeError(msg)
             conditions.append("%s=?" % column)
 
         query = "SELECT * FROM session WHERE %s" % (" AND ".join(conditions))
@@ -213,14 +421,14 @@ class SessionManager(LoggingConfigurable):
         if row is None:
             q = []
             for key, value in kwargs.items():
-                q.append("%s=%r" % (key, value))
+                q.append(f"{key}={value!r}")
 
             raise web.HTTPError(404, "Session not found: %s" % (", ".join(q)))
 
         try:
             model = await self.row_to_model(row)
         except KeyError as e:
-            raise web.HTTPError(404, "Session not found: %s" % str(e))
+            raise web.HTTPError(404, "Session not found: %s" % str(e)) from e
         return model
 
     async def update_session(self, session_id, **kwargs):
@@ -252,13 +460,13 @@ class SessionManager(LoggingConfigurable):
         query = "UPDATE session SET %s WHERE session_id=?" % (", ".join(sets))
         self.cursor.execute(query, list(kwargs.values()) + [session_id])
 
-    def kernel_culled(self, kernel_id):
+    async def kernel_culled(self, kernel_id: str) -> bool:
         """Checks if the kernel is still considered alive and returns true if its not found."""
         return kernel_id not in self.kernel_manager
 
     async def row_to_model(self, row, tolerate_culled=False):
         """Takes sqlite database session row and turns it into a dictionary"""
-        kernel_culled = await ensure_async(self.kernel_culled(row["kernel_id"]))
+        kernel_culled: bool = await ensure_async(self.kernel_culled(row["kernel_id"]))
         if kernel_culled:
             # The kernel was culled or died without deleting the session.
             # We can't use delete_session here because that tries to find
@@ -275,7 +483,7 @@ class SessionManager(LoggingConfigurable):
                 )
             )
             if tolerate_culled:
-                self.log.warning(msg + "  Continuing...")
+                self.log.warning(f"{msg}  Continuing...")
                 return
             raise KeyError(msg)
 
@@ -309,6 +517,9 @@ class SessionManager(LoggingConfigurable):
 
     async def delete_session(self, session_id):
         """Deletes the row in the session database with given session_id"""
+        record = KernelSessionRecord(session_id=session_id)
+        self._pending_sessions.update(record)
         session = await self.get_session(session_id=session_id)
         await ensure_async(self.kernel_manager.shutdown_kernel(session["kernel"]["id"]))
         self.cursor.execute("DELETE FROM session WHERE session_id=?", (session_id,))
+        self._pending_sessions.remove(record)
