@@ -8,16 +8,26 @@ import socket
 import sys
 import warnings
 from contextlib import contextmanager
-from urllib.parse import urljoin  # noqa: F401
-from urllib.parse import SplitResult, quote, unquote, urlparse, urlsplit, urlunsplit
+from typing import NewType
+from urllib.parse import (
+    SplitResult,
+    quote,
+    unquote,
+    urljoin,  # noqa
+    urlparse,
+    urlsplit,
+    urlunsplit,
+)
 from urllib.request import pathname2url  # noqa: F401
 
-from _frozen_importlib_external import _NamespacePath
-from jupyter_core.utils import ensure_async  # noqa: F401
+from _frozen_importlib_external import _NamespacePath  # type:ignore
+from jupyter_core.utils import ensure_async
 from packaging.version import Version
 from tornado.httpclient import AsyncHTTPClient, HTTPClient, HTTPRequest
 from tornado.netutil import Resolver
 
+ApiPath = NewType("ApiPath", str)
+
 
 def url_path_join(*pieces):
     """Join components of url into a relative url
@@ -102,19 +112,19 @@ def samefile_simple(path, other_path):
     return path.lower() == other_path.lower() and path_stat == other_path_stat
 
 
-def to_os_path(path, root=""):
+def to_os_path(path: ApiPath, root: str = "") -> str:
     """Convert an API path to a filesystem path
 
     If given, root will be prepended to the path.
     root must be a filesystem path already.
     """
-    parts = path.strip("/").split("/")
+    parts = str(path).strip("/").split("/")
     parts = [p for p in parts if p != ""]  # remove duplicate splits
-    path = os.path.join(root, *parts)
-    return os.path.normpath(path)
+    path_ = os.path.join(root, *parts)
+    return os.path.normpath(path_)
 
 
-def to_api_path(os_path, root=""):
+def to_api_path(os_path: str, root: str = "") -> ApiPath:
     """Convert a filesystem path to an API path
 
     If given, root will be removed from the path.
@@ -125,7 +135,7 @@ def to_api_path(os_path, root=""):
     parts = os_path.strip(os.path.sep).split(os.path.sep)
     parts = [p for p in parts if p != ""]  # remove duplicate splits
     path = "/".join(parts)
-    return path
+    return ApiPath(path)
 
 
 def check_version(v, check):
@@ -174,9 +184,11 @@ else:
 
 
 async def run_sync_in_loop(maybe_async):
-    """**DEPRECATED**: Use ``ensure_async`` instead."""
+    """**DEPRECATED**: Use ``ensure_async`` from jupyter_core instead."""
     warnings.warn(
-        "run_sync_in_loop is deprecated, use 'ensure_async'", DeprecationWarning, stacklevel=2
+        "run_sync_in_loop is deprecated since Jupyter Server 2.0, use 'ensure_async' from jupyter_core instead",
+        DeprecationWarning,
+        stacklevel=2,
     )
     return ensure_async(maybe_async)
 
@@ -255,7 +267,8 @@ def _request_for_tornado_client(urlstring, method="GET", body=None, headers=None
         resolver = UnixSocketResolver(resolver=Resolver())
         AsyncHTTPClient.configure(None, resolver=resolver)
     else:
-        raise Exception("Unknown URL scheme.")
+        msg = "Unknown URL scheme."
+        raise Exception(msg)
 
     # Yield the request for the given client.
     url = urlunsplit(parts)
@@ -351,7 +364,8 @@ def filefind(filename, path_dirs=None):
         if os.path.isfile(testname):
             return os.path.abspath(testname)
 
-    raise OSError(f"File {filename!r} does not exist in any of the search paths: {path_dirs!r}")
+    msg = f"File {filename!r} does not exist in any of the search paths: {path_dirs!r}"
+    raise OSError(msg)
 
 
 def expand_path(s):
@@ -395,8 +409,8 @@ def import_item(name):
         module = __import__(package, fromlist=[obj])
         try:
             pak = getattr(module, obj)
-        except AttributeError:
-            raise ImportError("No module named %s" % obj)
+        except AttributeError as e:
+            raise ImportError("No module named %s" % obj) from e
         return pak
     else:
         # called with un-dotted string
