@@ -264,14 +264,18 @@ def test_urls(config, public_url, local_url, connection_url):
 
 # Preferred dir tests
 # ----------------------------------------------------------------------------
+@pytest.mark.filterwarnings("ignore::FutureWarning")
 def test_valid_preferred_dir(tmp_path, jp_configurable_serverapp):
     path = str(tmp_path)
     app = jp_configurable_serverapp(root_dir=path, preferred_dir=path)
     assert app.root_dir == path
     assert app.preferred_dir == path
     assert app.root_dir == app.preferred_dir
+    assert app.contents_manager.root_dir == path
+    assert app.contents_manager.preferred_dir == ""
 
 
+@pytest.mark.filterwarnings("ignore::FutureWarning")
 def test_valid_preferred_dir_is_root_subdir(tmp_path, jp_configurable_serverapp):
     path = str(tmp_path)
     path_subdir = str(tmp_path / "subdir")
@@ -280,6 +284,7 @@ def test_valid_preferred_dir_is_root_subdir(tmp_path, jp_configurable_serverapp)
     assert app.root_dir == path
     assert app.preferred_dir == path_subdir
     assert app.preferred_dir.startswith(app.root_dir)
+    assert app.contents_manager.preferred_dir == "subdir"
 
 
 def test_valid_preferred_dir_does_not_exist(tmp_path, jp_configurable_serverapp):
@@ -291,26 +296,46 @@ def test_valid_preferred_dir_does_not_exist(tmp_path, jp_configurable_serverapp)
     assert "No such preferred dir:" in str(error)
 
 
+# This tests some deprecated behavior as well
+@pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize(
-    "root_dir_loc,preferred_dir_loc",
+    "root_dir_loc,preferred_dir_loc,config_target",
     [
-        ("cli", "cli"),
-        ("cli", "config"),
-        ("cli", "default"),
-        ("config", "cli"),
-        ("config", "config"),
-        ("config", "default"),
-        ("default", "cli"),
-        ("default", "config"),
-        ("default", "default"),
+        ("cli", "cli", "ServerApp"),
+        ("cli", "cli", "FileContentsManager"),
+        ("cli", "config", "ServerApp"),
+        ("cli", "config", "FileContentsManager"),
+        ("cli", "default", "ServerApp"),
+        ("cli", "default", "FileContentsManager"),
+        ("config", "cli", "ServerApp"),
+        ("config", "cli", "FileContentsManager"),
+        ("config", "config", "ServerApp"),
+        ("config", "config", "FileContentsManager"),
+        ("config", "default", "ServerApp"),
+        ("config", "default", "FileContentsManager"),
+        ("default", "cli", "ServerApp"),
+        ("default", "cli", "FileContentsManager"),
+        ("default", "config", "ServerApp"),
+        ("default", "config", "FileContentsManager"),
+        ("default", "default", "ServerApp"),
+        ("default", "default", "FileContentsManager"),
     ],
 )
 def test_preferred_dir_validation(
-    root_dir_loc, preferred_dir_loc, tmp_path, jp_config_dir, jp_configurable_serverapp
+    root_dir_loc,
+    preferred_dir_loc,
+    config_target,
+    tmp_path,
+    jp_config_dir,
+    jp_configurable_serverapp,
 ):
     expected_root_dir = str(tmp_path)
-    expected_preferred_dir = str(tmp_path / "subdir")
-    os.makedirs(expected_preferred_dir, exist_ok=True)
+
+    os_preferred_dir = str(tmp_path / "subdir")
+    os.makedirs(os_preferred_dir, exist_ok=True)
+    config_preferred_dir = os_preferred_dir if config_target == "ServerApp" else "subdir"
+    config_preferred_dir = config_preferred_dir + "/"  # add trailing slash to ensure it is removed
+    expected_preferred_dir = "subdir"
 
     argv = []
     kwargs = {"root_dir": None}
@@ -321,18 +346,18 @@ def test_preferred_dir_validation(
         config_file = jp_config_dir.joinpath("jupyter_server_config.py")
 
     if root_dir_loc == "cli":
-        argv.append(f"--ServerApp.root_dir={expected_root_dir}")
+        argv.append(f"--{config_target}.root_dir={expected_root_dir}")
     if root_dir_loc == "config":
-        config_lines.append(f'c.ServerApp.root_dir = r"{expected_root_dir}"')
+        config_lines.append(f'c.{config_target}.root_dir = r"{expected_root_dir}"')
     if root_dir_loc == "default":
         expected_root_dir = os.getcwd()
 
     if preferred_dir_loc == "cli":
-        argv.append(f"--ServerApp.preferred_dir={expected_preferred_dir}")
+        argv.append(f"--{config_target}.preferred_dir={config_preferred_dir}")
     if preferred_dir_loc == "config":
-        config_lines.append(f'c.ServerApp.preferred_dir = r"{expected_preferred_dir}"')
+        config_lines.append(f'c.{config_target}.preferred_dir = r"{config_preferred_dir}"')
     if preferred_dir_loc == "default":
-        expected_preferred_dir = expected_root_dir
+        expected_preferred_dir = ""
 
     if config_file is not None:
         config_file.write_text("\n".join(config_lines))
@@ -345,9 +370,9 @@ def test_preferred_dir_validation(
             jp_configurable_serverapp(**kwargs)
     else:
         app = jp_configurable_serverapp(**kwargs)
-        assert app.root_dir == expected_root_dir
-        assert app.preferred_dir == expected_preferred_dir
-        assert app.preferred_dir.startswith(app.root_dir)
+        assert app.contents_manager.root_dir == expected_root_dir
+        assert app.contents_manager.preferred_dir == expected_preferred_dir
+        assert ".." not in expected_preferred_dir
 
 
 def test_invalid_preferred_dir_does_not_exist(tmp_path, jp_configurable_serverapp):
@@ -370,45 +395,42 @@ def test_invalid_preferred_dir_does_not_exist_set(tmp_path, jp_configurable_serv
     assert "No such preferred dir:" in str(error)
 
 
+@pytest.mark.filterwarnings("ignore::FutureWarning")
 def test_invalid_preferred_dir_not_root_subdir(tmp_path, jp_configurable_serverapp):
     path = str(tmp_path / "subdir")
     os.makedirs(path, exist_ok=True)
     not_subdir_path = str(tmp_path)
 
+    with pytest.raises(SystemExit):
+        jp_configurable_serverapp(root_dir=path, preferred_dir=not_subdir_path)
+
+
+async def test_invalid_preferred_dir_not_root_subdir_set(tmp_path, jp_configurable_serverapp):
+    path = str(tmp_path / "subdir")
+    os.makedirs(path, exist_ok=True)
+    not_subdir_path = os.path.relpath(tmp_path, path)
+
+    app = jp_configurable_serverapp(root_dir=path)
     with pytest.raises(TraitError) as error:
-        app = jp_configurable_serverapp(root_dir=path, preferred_dir=not_subdir_path)
+        app.contents_manager.preferred_dir = not_subdir_path
 
-    assert "preferred_dir must be equal or a subdir of root_dir. " in str(error)
+    assert "is outside root contents directory" in str(error.value)
 
 
-def test_invalid_preferred_dir_not_root_subdir_set(tmp_path, jp_configurable_serverapp):
+async def test_absolute_preferred_dir_not_root_subdir_set(tmp_path, jp_configurable_serverapp):
     path = str(tmp_path / "subdir")
     os.makedirs(path, exist_ok=True)
     not_subdir_path = str(tmp_path)
 
     app = jp_configurable_serverapp(root_dir=path)
+
     with pytest.raises(TraitError) as error:
-        app.preferred_dir = not_subdir_path
+        app.contents_manager.preferred_dir = not_subdir_path
 
-    assert "preferred_dir must be equal or a subdir of root_dir. " in str(error)
-
-
-def test_observed_root_dir_updates_preferred_dir(tmp_path, jp_configurable_serverapp):
-    path = str(tmp_path)
-    new_path = str(tmp_path / "subdir")
-    os.makedirs(new_path, exist_ok=True)
-
-    app = jp_configurable_serverapp(root_dir=path, preferred_dir=path)
-    app.root_dir = new_path
-    assert app.preferred_dir == new_path
-
-
-def test_observed_root_dir_does_not_update_preferred_dir(tmp_path, jp_configurable_serverapp):
-    path = str(tmp_path)
-    new_path = str(tmp_path.parent)
-    app = jp_configurable_serverapp(root_dir=path, preferred_dir=path)
-    app.root_dir = new_path
-    assert app.preferred_dir == path
+    if os.name == "nt":
+        assert "is not a relative API path" in str(error.value)
+    else:
+        assert "Preferred directory not found" in str(error.value)
 
 
 def test_random_ports():
@@ -447,13 +469,13 @@ def test_misc(jp_serverapp, tmp_path):
     app.parse_command_line([])
 
 
-def test_deprecated_props(jp_serverapp):
+def test_deprecated_props(jp_serverapp, tmp_path):
     app: ServerApp = jp_serverapp
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         app.cookie_options = dict(foo=1)
         app.get_secure_cookie_kwargs = dict(bar=1)
-        app.notebook_dir = "foo"
+        app.notebook_dir = str(tmp_path)
         app.server_extensions = dict(foo=True)
         app.kernel_ws_protocol = "foo"
         app.limit_rate = True
